"use strict";

let websocket = require("websocket");
let {http} = require("./HTTPServer");
let {randomUUID} = require("crypto");
const { Client } = require("./Client.js");

console.log("Web Socket Protocol is ready");

http.addListener("upgrade",() => {
    console.log("HTTP Upgrading to WebSocket");
})

let wsServer = new websocket.server({
    httpServer: http,
    autoAcceptConnections: true
});

let global = new Map();
let clients = new Map();
wsServer.addListener("connect",(socket) => {

    let xClient = new Client();
    let id = randomUUID();
    socket.id = id;
    xClient.id = id;
    xClient.socket = socket;
    xClient.created_at = new Date();
    Client.clients.set(id, xClient);
    clients.set(id, xClient);

    emit("connect", global, xClient);
    socket.addListener("close",()=>{
        emit("disconnect",  global, xClient);
        Client.clients.set(id, xClient);
    });
    socket.addListener("message",({type,utf8Data}) => {
        if(type == "utf8")
        {
            let json;
            try{
                json = JSON.parse(utf8Data);
                emit('services', global, xClient, json);
                let [payload, id, action] = json;
                if(typeof id === "string")
                {
                    action = id;
                }
                emitService(global, xClient, id, payload, action);
            }catch{
                emit("messageError", global, xClient, utf8Data);
            }
        }
    });
});

/**
 * @type {Map<string, Function[]>}
 */
 let events = new Map();
 /**
  * @type {Map<string, Function[]>}
  */
 let services = []
/**
 * 
 * @param {string} event 
 * @param {(global:Map<string, any>, client:Client, data:any) => any} func 
 */
exports.addListener = (event, func) => {
    if(!events.has(event))
    {
        events.set(event,[]);
    };
    events.get(event).push(func);
};
/**
 * 
 * @param {string} event 
 * @param {(data:{global:Map<string, any>, client:Client, message:any,response:Function,end:Function,next:Function}) => any} func 
 */
exports.addService = (func) => {
    services.push(func);
};
function emit(event,...args)
{
    if(events.has(event))
    {
        for (const callback of events.get(event)) {
            callback(...args);
        }
    };
};
/**
 * 
 * @param {Map} global 
 * @param {Client} local 
 * @param {number} id 
 * @param {{[key:string]:any}} payload 
 * @param {"R"|"S"} action [R]EQUEST flag or [S]TREAM flag
 */
async function emitService(global, client, id, payload, action)
{
    let willContinue = false;
    for (const callback of services) {
        await callback({
            message: payload,
            action,
            client,
            global,
            response:(obj)=>{
                client.send([obj, id, 'C']) // continue ([C]ONTINUE flag)
            },
            end:(obj)=>{
                client.send([obj, id, 'E']) // stopped data stream (this channel) ([E]ND flag)
            },
            next:function(){
                willContinue = true;
            }
        });
        if(willContinue === false) break;
    }
};
exports.websocket = wsServer;